import { Building, planElectricalLines } from './solver.js';

function buildingsByCoordinates(firstBuilding, secondBuilding) {
  if (firstBuilding.x !== secondBuilding.x) {
    return firstBuilding.x - secondBuilding.x;
  }
  return firstBuilding.y - secondBuilding.y;
}

function linesByCoordinates([firstBuilding, secondBuilding], [firstOtherBuilding, secondOtherBuilding]) {
  const candidate = buildingsByCoordinates(firstBuilding, firstOtherBuilding);
  if (candidate !== 0) {
    return candidate;
  }
  return buildingsByCoordinates(secondBuilding, secondOtherBuilding);
}

function canonicalizeNetwork(network) {
  return network.map(
    (line) => [line.firstBuilding, line.secondBuilding].sort(buildingsByCoordinates),
  ).sort(linesByCoordinates);
}

describe('planElectricalLines minimizes cable length', () => {
  test('when there are no buildings', () => {
    expect(canonicalizeNetwork(planElectricalLines([]))).toEqual([]);
  });
  test('when there is one building', () => {
    const a = new Building(0, 0);
    expect(canonicalizeNetwork(planElectricalLines([
      a,
    ]))).toEqual([]);
  });
  test('when there are two buildings', () => {
    const a = new Building(0, 0);
    const b = new Building(0, 1);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b,
    ]))).toEqual([
      [a, b],
    ]);
  });
  test('when there are three buildings', () => {
    const a = new Building(0, 0);
    const b = new Building(0, 1);
    const c = new Building(1, 0);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c,
    ]))).toEqual([
      [a, b],
      [a, c],
    ]);
  });
  test('when there are four buildings with none isolated', () => {
    const a = new Building(0, 0);
    const b = new Building(0, 1);
    const c = new Building(4, 0);
    const d = new Building(5, 1);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d,
    ]))).toEqual([
      [a, b],
      [a, c],
      [c, d],
    ]);
  });
  test('when there are four buildings with one isolated', () => {
    const a = new Building(0, 0);
    const b = new Building(0, 1);
    const c = new Building(4, 0);
    const d = new Building(3, 5);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d,
    ]))).toEqual([
      [a, b],
      [a, c],
      [b, d],
    ]);
  });
  test('on random imput #0', () => {
    const a = new Building(0, 4);
    const b = new Building(1, 5);
    const c = new Building(2, 7);
    const d = new Building(5, 8);
    const e = new Building(7, 3);
    const f = new Building(8, 8);
    const g = new Building(8, 9);
    const h = new Building(9, 1);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h,
    ]))).toEqual([
      [a, b],
      [b, c],
      [c, d],
      [d, f],
      [e, f],
      [e, h],
      [f, g],
    ]);
  });
  test('on random imput #1', () => {
    const a = new Building(0, 8);
    const b = new Building(1, 0);
    const c = new Building(1, 3);
    const d = new Building(4, 7);
    const e = new Building(5, 5);
    const f = new Building(5, 6);
    const g = new Building(7, 3);
    const h = new Building(8, 8);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h,
    ]))).toEqual([
      [a, d],
      [b, c],
      [c, e],
      [d, f],
      [e, f],
      [e, g],
      [f, h],
    ]);
  });
  test('on random imput #2', () => {
    const a = new Building(0, 2);
    const b = new Building(0, 4);
    const c = new Building(1, 0);
    const d = new Building(1, 4);
    const e = new Building(5, 8);
    const f = new Building(6, 5);
    const g = new Building(7, 6);
    const h = new Building(8, 2);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h,
    ]))).toEqual([
      [a, b],
      [a, c],
      [b, d],
      [d, f],
      [e, g],
      [f, g],
      [f, h],
    ]);
  });
  test('on random imput #3', () => {
    const a = new Building(3, 3);
    const b = new Building(4, 0);
    const c = new Building(4, 5);
    const d = new Building(5, 9);
    const e = new Building(7, 2);
    const f = new Building(7, 9);
    const g = new Building(8, 1);
    const h = new Building(9, 4);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h,
    ]))).toEqual([
      [a, b],
      [a, c],
      [b, e],
      [c, d],
      [d, f],
      [e, g],
      [e, h],
    ]);
  });
  test('on random imput #4', () => {
    const a = new Building(0, 7);
    const b = new Building(2, 1);
    const c = new Building(3, 6);
    const d = new Building(4, 7);
    const e = new Building(6, 0);
    const f = new Building(8, 5);
    const g = new Building(9, 2);
    const h = new Building(9, 3);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h,
    ]))).toEqual([
      [a, c],
      [b, e],
      [c, d],
      [d, f],
      [e, g],
      [f, h],
      [g, h],
    ]);
  });
  test('on random imput #5', () => {
    const a = new Building(0, 18);
    const b = new Building(1, 15);
    const c = new Building(2, 1);
    const d = new Building(3, 8);
    const e = new Building(4, 4);
    const f = new Building(7, 10);
    const g = new Building(11, 19);
    const h = new Building(12, 1);
    const i = new Building(13, 16);
    const j = new Building(13, 19);
    const k = new Building(15, 7);
    const l = new Building(15, 13);
    const m = new Building(15, 14);
    const n = new Building(16, 12);
    const o = new Building(17, 1);
    const p = new Building(17, 19);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p,
    ]))).toEqual([
      [a, b],
      [b, d],
      [c, e],
      [d, e],
      [d, f],
      [f, i],
      [g, j],
      [h, o],
      [i, j],
      [i, m],
      [j, p],
      [k, n],
      [k, o],
      [l, m],
      [l, n],
    ]);
  });
  test('on random imput #6', () => {
    const a = new Building(0, 4);
    const b = new Building(1, 17);
    const c = new Building(3, 4);
    const d = new Building(4, 4);
    const e = new Building(4, 8);
    const f = new Building(5, 19);
    const g = new Building(6, 0);
    const h = new Building(6, 2);
    const i = new Building(8, 11);
    const j = new Building(13, 15);
    const k = new Building(15, 11);
    const l = new Building(15, 18);
    const m = new Building(17, 4);
    const n = new Building(17, 12);
    const o = new Building(18, 15);
    const p = new Building(19, 16);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p,
    ]))).toEqual([
      [a, c],
      [b, f],
      [c, d],
      [d, e],
      [d, h],
      [e, i],
      [f, i],
      [g, h],
      [i, j],
      [j, l],
      [k, m],
      [k, n],
      [l, o],
      [n, o],
      [o, p],
    ]);
  });
  test('on random imput #7', () => {
    const a = new Building(0, 12);
    const b = new Building(0, 18);
    const c = new Building(1, 7);
    const d = new Building(3, 0);
    const e = new Building(3, 7);
    const f = new Building(6, 0);
    const g = new Building(7, 14);
    const h = new Building(7, 19);
    const i = new Building(9, 18);
    const j = new Building(10, 3);
    const k = new Building(13, 10);
    const l = new Building(13, 18);
    const m = new Building(15, 0);
    const n = new Building(16, 9);
    const o = new Building(17, 5);
    const p = new Building(18, 6);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p,
    ]))).toEqual([
      [a, b],
      [a, c],
      [b, h],
      [c, e],
      [d, e],
      [d, f],
      [f, j],
      [g, i],
      [h, i],
      [i, l],
      [j, m],
      [k, n],
      [m, o],
      [n, p],
      [o, p],
    ]);
  });
  test('on random imput #8', () => {
    const a = new Building(0, 19);
    const b = new Building(3, 14);
    const c = new Building(4, 6);
    const d = new Building(5, 10);
    const e = new Building(7, 18);
    const f = new Building(8, 17);
    const g = new Building(10, 17);
    const h = new Building(10, 18);
    const i = new Building(12, 1);
    const j = new Building(14, 3);
    const k = new Building(14, 11);
    const l = new Building(16, 0);
    const m = new Building(16, 12);
    const n = new Building(18, 18);
    const o = new Building(19, 1);
    const p = new Building(19, 4);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p,
    ]))).toEqual([
      [a, b],
      [b, d],
      [b, e],
      [c, d],
      [e, f],
      [f, g],
      [g, h],
      [g, k],
      [i, j],
      [j, k],
      [j, l],
      [k, m],
      [l, o],
      [m, n],
      [o, p],
    ]);
  });
  test('on random imput #9', () => {
    const a = new Building(0, 7);
    const b = new Building(1, 0);
    const c = new Building(1, 9);
    const d = new Building(4, 3);
    const e = new Building(4, 6);
    const f = new Building(6, 13);
    const g = new Building(6, 15);
    const h = new Building(7, 5);
    const i = new Building(7, 15);
    const j = new Building(8, 16);
    const k = new Building(12, 4);
    const l = new Building(12, 11);
    const m = new Building(17, 14);
    const n = new Building(17, 19);
    const o = new Building(19, 5);
    const p = new Building(19, 12);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p,
    ]))).toEqual([
      [a, c],
      [a, e],
      [b, d],
      [c, f],
      [d, e],
      [e, h],
      [f, g],
      [f, l],
      [g, i],
      [h, k],
      [i, j],
      [l, m],
      [m, n],
      [m, p],
      [o, p],
    ]);
  });
  test('on random imput #10', () => {
    const a = new Building(0, 2);
    const b = new Building(0, 15);
    const c = new Building(1, 11);
    const d = new Building(1, 23);
    const e = new Building(4, 9);
    const f = new Building(4, 29);
    const g = new Building(6, 21);
    const h = new Building(7, 9);
    const i = new Building(8, 15);
    const j = new Building(11, 24);
    const k = new Building(14, 2);
    const l = new Building(16, 13);
    const m = new Building(17, 12);
    const n = new Building(18, 24);
    const o = new Building(19, 1);
    const p = new Building(20, 25);
    const q = new Building(21, 12);
    const r = new Building(23, 24);
    const s = new Building(24, 24);
    const t = new Building(28, 20);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, q, r, s, t,
    ]))).toEqual([
      [a, e],
      [b, c],
      [c, e],
      [d, f],
      [d, g],
      [e, h],
      [g, i],
      [g, j],
      [h, i],
      [h, k],
      [i, l],
      [j, n],
      [k, o],
      [l, m],
      [m, q],
      [n, p],
      [p, r],
      [r, s],
      [s, t],
    ]);
  });
  test('on random imput #11', () => {
    const a = new Building(0, 21);
    const b = new Building(1, 27);
    const c = new Building(2, 15);
    const d = new Building(3, 2);
    const e = new Building(3, 18);
    const f = new Building(10, 18);
    const g = new Building(11, 10);
    const h = new Building(13, 13);
    const i = new Building(13, 27);
    const j = new Building(19, 1);
    const k = new Building(19, 11);
    const l = new Building(21, 1);
    const m = new Building(21, 2);
    const n = new Building(21, 16);
    const o = new Building(22, 3);
    const p = new Building(22, 21);
    const q = new Building(24, 4);
    const r = new Building(25, 8);
    const s = new Building(25, 12);
    const t = new Building(29, 26);
    expect(canonicalizeNetwork(planElectricalLines([
      a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, q, r, s, t,
    ]))).toEqual([
      [a, b],
      [a, e],
      [c, e],
      [d, g],
      [e, f],
      [f, h],
      [f, i],
      [g, h],
      [h, k],
      [j, l],
      [k, n],
      [l, m],
      [m, o],
      [n, p],
      [n, s],
      [o, q],
      [p, t],
      [q, r],
      [r, s],
    ]);
  });
});
